"""Sample of using the menu component."""

from material_ui._component import Component, effect, use_state
from material_ui.buttons import FilledButton
from material_ui.icon import Icon
from material_ui.layout_basics import Stack
from material_ui.menu import Menu, MenuItem
from material_ui.tokens import md_sys_color
from material_ui.typography import Typography
from qtpy.QtCore import QMargins, Qt
from qtpy.QtWidgets import QApplication


class SampleMenu(Component):
    selected_item = use_state("")

    def __init__(self) -> None:
        super().__init__()
        self.sx = {"background-color": md_sys_color.background}

        stack = Stack()
        stack.gap = 20
        stack.margins = QMargins(40, 40, 40, 40)
        stack.alignment = Qt.AlignmentFlag.AlignCenter
        self.overlay_widget(stack)

        self._show_menu_button = FilledButton()
        self._show_menu_button.text = "Open Menu"
        self._show_menu_button.clicked.connect(self._on_click_show_menu_button)
        stack.add_widget(self._show_menu_button)

        self._selected_label = Typography()
        self._selected_label.alignment = Qt.AlignmentFlag.AlignCenter
        stack.add_widget(self._selected_label)

    def _on_click_show_menu_button(self) -> None:
        menu = Menu()

        item1 = MenuItem()
        item1.text = "Item 1"
        item1_icon = Icon()
        item1_icon.icon_name = "check"
        item1.leading_icon = item1_icon
        item1.clicked.connect(self._on_click_item1)
        item1.setParent(menu)

        item2 = MenuItem()
        item2.text = "Item 2"
        item2.clicked.connect(self._on_click_item2)
        item2.setParent(menu)

        item3 = MenuItem()
        item3.text = "Item 3"
        item3.clicked.connect(self._on_click_item3)
        item3.setParent(menu)

        menu.open(anchor_widget=self._show_menu_button)

    def _on_click_item1(self) -> None:
        self.selected_item = "Item 1"

    def _on_click_item2(self) -> None:
        self.selected_item = "Item 2"

    def _on_click_item3(self) -> None:
        self.selected_item = "Item 3"

    @effect(selected_item)
    def _apply_selected_label_text(self) -> None:
        new_text = (
            "No selection"
            if not self.selected_item
            else f"Selected: {self.selected_item}"
        )
        self._selected_label.text = new_text


def main() -> None:
    app = QApplication()
    window = SampleMenu()
    window.show()
    app.exec_()


if __name__ == "__main__":
    main()



"""Design tokens for md.sys.state."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

dragged_state_layer_opacity = _define_token(0.16)
focus_indicator_inner_offset = _define_token(0)
focus_indicator_outer_offset = _define_token(2)
focus_indicator_thickness = _define_token(3)
focus_state_layer_opacity = _define_token(0.1)
hover_state_layer_opacity = _define_token(0.08)
pressed_state_layer_opacity = _define_token(0.1)



"""Design tokens for md.comp.list."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

divider_color = _define_token("md.sys.color.outline")
divider_height = _define_token(1)
divider_leading_space = _define_token(16)
divider_trailing_space = _define_token(16)
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.inner-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
list_item_container_color = _define_token("md.sys.color.surface")
list_item_container_elevation = _define_token("md.sys.elevation.level0")
list_item_container_height = _define_token(56)
list_item_container_shape = _define_token("md.sys.shape.corner.none")
list_item_disabled_label_text_color = _define_token("md.sys.color.on-surface")
list_item_disabled_label_text_opacity = _define_token(0.38)
list_item_disabled_leading_icon_color = _define_token("md.sys.color.on-surface")
list_item_disabled_leading_icon_opacity = _define_token(0.38)
list_item_disabled_state_layer_color = _define_token("md.sys.color.on-surface")
list_item_disabled_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
list_item_disabled_trailing_icon_color = _define_token("md.sys.color.on-surface")
list_item_disabled_trailing_icon_opacity = _define_token(0.38)
list_item_dragged_container_elevation = _define_token("md.sys.elevation.level4")
list_item_dragged_label_text_color = _define_token("md.sys.color.on-surface")
list_item_dragged_leading_icon_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_dragged_state_layer_color = _define_token("md.sys.color.on-surface")
list_item_dragged_state_layer_opacity = _define_token("md.sys.state.dragged.state-layer-opacity")
list_item_dragged_trailing_icon_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_focus_label_text_color = _define_token("md.sys.color.on-surface")
list_item_focus_leading_icon_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_focus_state_layer_color = _define_token("md.sys.color.on-surface")
list_item_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
list_item_focus_trailing_icon_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_hover_label_text_color = _define_token("md.sys.color.on-surface")
list_item_hover_leading_icon_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_hover_state_layer_color = _define_token("md.sys.color.on-surface")
list_item_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
list_item_hover_trailing_icon_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_label_text_color = _define_token("md.sys.color.on-surface")
list_item_label_text_font = _define_token("md.sys.typescale.body-large.font")
list_item_label_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
list_item_label_text_size = _define_token("md.sys.typescale.body-large.size")
list_item_label_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
list_item_label_text_weight = _define_token("md.sys.typescale.body-large.weight")
list_item_large_leading_video_height = _define_token(69)
list_item_leading_avatar_label_color = _define_token("md.sys.color.on-primary-container")
list_item_leading_avatar_label_font = _define_token("md.sys.typescale.title-medium.font")
list_item_leading_avatar_label_line_height = _define_token("md.sys.typescale.title-medium.line-height")
list_item_leading_avatar_label_size = _define_token("md.sys.typescale.title-medium.size")
list_item_leading_avatar_label_tracking = _define_token("md.sys.typescale.title-medium.tracking")
list_item_leading_avatar_label_weight = _define_token("md.sys.typescale.title-medium.weight")
list_item_leading_avatar_color = _define_token("md.sys.color.primary-container")
list_item_leading_avatar_shape = _define_token("md.sys.shape.corner.full")
list_item_leading_avatar_size = _define_token(40)
list_item_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_leading_icon_size = _define_token(24)
list_item_leading_image_height = _define_token(56)
list_item_leading_image_shape = _define_token("md.sys.shape.corner.none")
list_item_leading_image_width = _define_token(56)
list_item_leading_space = _define_token(16)
list_item_leading_video_shape = _define_token("md.sys.shape.corner.none")
list_item_leading_video_width = _define_token(100)
list_item_one_line_container_height = _define_token(56)
list_item_overline_color = _define_token("md.sys.color.on-surface-variant")
list_item_overline_font = _define_token("md.sys.typescale.label-small.font")
list_item_overline_line_height = _define_token("md.sys.typescale.label-small.line-height")
list_item_overline_size = _define_token("md.sys.typescale.label-small.size")
list_item_overline_tracking = _define_token("md.sys.typescale.label-small.tracking")
list_item_overline_weight = _define_token("md.sys.typescale.label-small.weight")
list_item_pressed_label_text_color = _define_token("md.sys.color.on-surface")
list_item_pressed_leading_icon_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_pressed_state_layer_color = _define_token("md.sys.color.on-surface")
list_item_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
list_item_pressed_trailing_icon_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_selected_container_color = _define_token("md.sys.color.secondary-container")
list_item_selected_dragged_label_text_color = _define_token("md.sys.color.on-secondary-container")
list_item_selected_dragged_leading_icon_color = _define_token("md.sys.color.on-surface")
list_item_selected_dragged_state_layer_color = _define_token("md.sys.color.on-surface")
list_item_selected_dragged_state_layer_opacity = _define_token("md.sys.state.dragged.state-layer-opacity")
list_item_selected_dragged_trailing_icon_color = _define_token("md.sys.color.on-surface")
list_item_selected_focus_label_text_color = _define_token("md.sys.color.on-secondary-container")
list_item_selected_focus_leading_icon_color = _define_token("md.sys.color.on-surface")
list_item_selected_focus_state_layer_color = _define_token("md.sys.color.on-surface")
list_item_selected_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
list_item_selected_focus_trailing_icon_color = _define_token("md.sys.color.on-surface")
list_item_selected_hover_label_text_color = _define_token("md.sys.color.on-secondary-container")
list_item_selected_hover_leading_icon_color = _define_token("md.sys.color.on-surface")
list_item_selected_hover_state_layer_color = _define_token("md.sys.color.on-surface")
list_item_selected_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
list_item_selected_hover_trailing_icon_color = _define_token("md.sys.color.on-surface")
list_item_selected_label_text_color = _define_token("md.sys.color.on-secondary-container")
list_item_selected_leading_icon_color = _define_token("md.sys.color.on-surface")
list_item_selected_overline_color = _define_token("md.sys.color.on-surface")
list_item_selected_pressed_label_text_color = _define_token("md.sys.color.on-secondary-container")
list_item_selected_pressed_leading_icon_color = _define_token("md.sys.color.on-surface")
list_item_selected_pressed_state_layer_color = _define_token("md.sys.color.on-surface")
list_item_selected_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
list_item_selected_pressed_trailing_icon_color = _define_token("md.sys.color.on-surface")
list_item_selected_supporting_text_color = _define_token("md.sys.color.on-surface")
list_item_selected_trailing_icon_color = _define_token("md.sys.color.primary")
list_item_selected_trailing_supporting_text_color = _define_token("md.sys.color.on-surface")
list_item_small_leading_video_height = _define_token(56)
list_item_supporting_text_color = _define_token("md.sys.color.on-surface-variant")
list_item_supporting_text_font = _define_token("md.sys.typescale.body-medium.font")
list_item_supporting_text_line_height = _define_token("md.sys.typescale.body-medium.line-height")
list_item_supporting_text_size = _define_token("md.sys.typescale.body-medium.size")
list_item_supporting_text_tracking = _define_token("md.sys.typescale.body-medium.tracking")
list_item_supporting_text_weight = _define_token("md.sys.typescale.body-medium.weight")
list_item_three_line_container_height = _define_token(88)
list_item_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_trailing_icon_size = _define_token(24)
list_item_trailing_space = _define_token(16)
list_item_trailing_supporting_text_color = _define_token("md.sys.color.on-surface-variant")
list_item_trailing_supporting_text_font = _define_token("md.sys.typescale.label-small.font")
list_item_trailing_supporting_text_line_height = _define_token("md.sys.typescale.label-small.line-height")
list_item_trailing_supporting_text_size = _define_token("md.sys.typescale.label-small.size")
list_item_trailing_supporting_text_tracking = _define_token("md.sys.typescale.label-small.tracking")
list_item_trailing_supporting_text_weight = _define_token("md.sys.typescale.label-small.weight")
list_item_two_line_container_height = _define_token(72)
list_item_unselected_trailing_icon_color = _define_token("md.sys.color.on-surface")
md_comp_list_list_item_one_line_container_height = _define_token(56)
md_comp_list_list_item_three_line_container_height = _define_token(88)
md_comp_list_list_item_two_line_container_height = _define_token(72)



"""Utilities."""

from collections.abc import Mapping

from qtpy.QtGui import QColor

from material_ui.tokens import DesignToken, resolve_token

StyleDictValue = str | int | float | QColor | DesignToken
"""Union of values that can be used in a style dictionary."""

# TODO: use a typed dict for completion on keys and type checking on values
StyleDict = Mapping[str, StyleDictValue]
"""Dictionary of styles."""


def convert_sx_to_qss(sx: StyleDict) -> str:
    """Convert a style dictionary to a Qt Style Sheet string.

    Args:
        sx: System property value.

    Returns:
        QSS string.
    """
    return ";".join(f"{key}:{_stringify_sx_value(value)}" for key, value in sx.items())


def _stringify_sx_value(value: StyleDictValue) -> str:
    """Convert a value to a string.

    Design tokens are resolved to the underlying values.

    Args:
        value: Value to convert.

    Returns:
        String representation of the value.
    """
    # First things first, resolve the design tokens.
    if isinstance(value, DesignToken):
        value = resolve_token(value)
    # Then convert the special values to strings.
    if isinstance(value, QColor):
        return f"rgba({value.red()},{value.green()},{value.blue()},{value.alpha()})"
    if isinstance(value, int):
        return f"{value}px"
    if isinstance(value, float):
        return f"{value}"
    return value


undefined = object()
"""A special token to indicate an undefined value."""



"""Token core."""

import contextlib
import re
from dataclasses import dataclass
from functools import partial
from typing import TypeVar, cast

from qtpy.QtGui import QColor

Indirection = str
"""Token value that is a reference to another token."""


TokenValue = QColor | float | int
"""Union of underlying token value types (excluding Indirections)."""


@dataclass
class DesignToken:
    """Token runtime value wrapper type."""

    value: Indirection | TokenValue

    def __hash__(self) -> int:
        # Allow the token to be used as a dictionary key.
        # TODO: fix it to use @dataclass(unsafe_hash=True) instead
        # QColor isn't hashable, so convert to string
        return hash(repr(self.value))


_token_registry: set[DesignToken] = set()
"""Global registry for all registered tokens."""


def define_token(value: TokenValue | Indirection) -> DesignToken:
    """Factory function for defining a token and registering it by name.

    Mainly for internal use.
    """
    ret_val = DesignToken(value)
    _token_registry.add(ret_val)
    return ret_val


def resolve_token(token: DesignToken) -> TokenValue:
    """Resolve a token to its underlying value.

    If there are multiple token indirections, they are recursively
    resolved until a value is obtained.

    Args:
        token: The token to resolve.

    Returns:
        The value represented by the token.

    Example:
        from material_ui.tokens import md_comp_elevated_button as tokens
        value = resolve_token(tokens.container_color)
    """
    # Assume the root token is a value not indirection.
    return cast("TokenValue", find_root_token(token).value)


_T = TypeVar("_T")


def resolve_token_or_value(token_or_value: _T | DesignToken) -> _T:
    """If a token is passed, resolve it. Otherwise, return the value itself."""
    if isinstance(token_or_value, DesignToken):
        resolved = resolve_token(token_or_value)
        return cast("_T", resolved)
    return token_or_value


def find_root_token(token: DesignToken) -> DesignToken:
    """Resolve a token to the last token in the chain of indirection.

    Consider `resolve_token` if just trying to get the underlying token
    value. This function is mainly useful for using tokens as hash keys.

    Args:
        token: The token to resolve.

    Returns:
        The last token in the chain of indirections.
    """
    # Because the tokens system stores variables as values or names of
    # other tokens, first check if the tokens is a value already.
    if not _is_indirection(token):
        return token
    indirection = _resolve_indirection(cast("Indirection", token.value))
    if not indirection:
        msg = f"Unable to resolve token indirection: {token.value}"
        raise ValueError(msg)
    # Continue recursively to check until a value is obtained.
    return find_root_token(indirection)


# TODO: make proper indirection class instead of hardcoded string check
_NOT_INDIRECTION = {"SHAPE_FAMILY_CIRCULAR", "SHAPE_FAMILY_ROUNDED_CORNERS", "Roboto"}


def _is_indirection(token: DesignToken) -> bool:
    """Check if the token is an indirection.

    Args:
        token: The token to check.

    Returns:
        True if the token is an indirection, False if a value.
    """
    is_value_type = isinstance(token.value, TokenValue)
    return not is_value_type and token.value not in _NOT_INDIRECTION


def _resolve_indirection(value: Indirection) -> DesignToken | None:
    if match_result := re.match(r"(md\.(?:ref|comp|sys)\..+?)\.(.*)", value):
        py_module_name = to_python_name(match_result.group(1))
        var_name = to_python_name(match_result.group(2))
        with contextlib.suppress(ImportError):
            module = __import__(f"material_ui.tokens.{py_module_name}")
            with contextlib.suppress(AttributeError):
                token = getattr(getattr(module.tokens, py_module_name), var_name)
                if isinstance(token, DesignToken):
                    return token
    return None


def override_token(token: DesignToken, value: TokenValue) -> None:
    """Override a token value in the global theme and notify listeners."""
    token.value = value
    # Local import to avoid circular import.
    from material_ui.theming.theme_hook import ThemeHook

    ThemeHook.get().on_change.emit()


to_python_name = partial(re.sub, r"[-\.]", "_")
"""Convert a token name to a valid Python identifier.

    Eg, md.comp.elevated-button.container-color ->
    md_comp_elevated_button_container_color
"""



"""Design tokens for md.comp.plain-tooltip."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

container_color = _define_token("md.sys.color.inverse-surface")
container_shape = _define_token("md.sys.shape.corner.extra-small")
supporting_text_color = _define_token("md.sys.color.inverse-on-surface")
supporting_text_font = _define_token("md.sys.typescale.body-small.font")
supporting_text_line_height = _define_token("md.sys.typescale.body-small.line-height")
supporting_text_size = _define_token("md.sys.typescale.body-small.size")
supporting_text_tracking = _define_token("md.sys.typescale.body-small.tracking")
supporting_text_weight = _define_token("md.sys.typescale.body-small.weight")



"""Base progress."""

from material_ui._component import Component, use_state


class BaseProgress(Component):
    """Base class for progress indicators."""

    value = use_state(0.0)
    """Progress percentage, in range [0, 1]."""

    indeterminate = use_state(False)
    """Whether the progress is indeterminate (looping animation)."""



"""Sample of using the elevation."""

from qtpy import QtCore, QtWidgets
from material_ui.elevation import Elevation
from material_ui.layout_basics import Row
from material_ui.tokens import md_sys_elevation
from material_ui.typography import Typography


def main() -> None:
    """Main function."""
    app = QtWidgets.QApplication()

    window = Row()
    window.alignment.set(QtCore.Qt.AlignCenter)
    window.gap.set(20)
    window.sx.set({"background-color": "white"})
    window.resize(400, 200)

    levels = [
        md_sys_elevation.level0,
        md_sys_elevation.level1,
        md_sys_elevation.level2,
        md_sys_elevation.level3,
        md_sys_elevation.level4,
    ]
    for i, level in enumerate(levels):
        comp = Elevation()
        comp.setFixedSize(50, 50)
        comp.elevation.set(level)

        label = Typography()
        label.text.set(str(i))
        label.setParent(comp)
        label.move(5, 5)

        window.add_widget(comp)

    window.show()
    app.exec_()


if __name__ == "__main__":
    main()



"""Design tokens for md.comp.dialog."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

action_focus_label_text_color = _define_token("md.sys.color.primary")
action_focus_state_layer_color = _define_token("md.sys.color.primary")
action_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
action_hover_label_text_color = _define_token("md.sys.color.primary")
action_hover_state_layer_color = _define_token("md.sys.color.primary")
action_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
action_label_text_color = _define_token("md.sys.color.primary")
action_label_text_font = _define_token("md.sys.typescale.label-large.font")
action_label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
action_label_text_size = _define_token("md.sys.typescale.label-large.size")
action_label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
action_label_text_weight = _define_token("md.sys.typescale.label-large.weight")
action_pressed_label_text_color = _define_token("md.sys.color.primary")
action_pressed_state_layer_color = _define_token("md.sys.color.primary")
action_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
container_color = _define_token("md.sys.color.surface-container-high")
container_elevation = _define_token("md.sys.elevation.level3")
container_shape = _define_token("md.sys.shape.corner.extra-large")
container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
headline_color = _define_token("md.sys.color.on-surface")
headline_font = _define_token("md.sys.typescale.headline-small.font")
headline_line_height = _define_token("md.sys.typescale.headline-small.line-height")
headline_size = _define_token("md.sys.typescale.headline-small.size")
headline_tracking = _define_token("md.sys.typescale.headline-small.tracking")
headline_weight = _define_token("md.sys.typescale.headline-small.weight")
subhead_color = _define_token("md.sys.color.on-surface")
subhead_font = _define_token("md.sys.typescale.headline-small.font")
subhead_line_height = _define_token("md.sys.typescale.headline-small.line-height")
subhead_size = _define_token("md.sys.typescale.headline-small.size")
subhead_tracking = _define_token("md.sys.typescale.headline-small.tracking")
subhead_weight = _define_token("md.sys.typescale.headline-small.weight")
supporting_text_color = _define_token("md.sys.color.on-surface-variant")
supporting_text_font = _define_token("md.sys.typescale.body-medium.font")
supporting_text_line_height = _define_token("md.sys.typescale.body-medium.line-height")
supporting_text_size = _define_token("md.sys.typescale.body-medium.size")
supporting_text_tracking = _define_token("md.sys.typescale.body-medium.tracking")
supporting_text_weight = _define_token("md.sys.typescale.body-medium.weight")
with_divider_divider_color = _define_token("md.sys.color.outline")
with_divider_divider_height = _define_token(1)
with_icon_icon_color = _define_token("md.sys.color.secondary")
with_icon_icon_size = _define_token(24)



"""Design tokens for md.sys.elevation."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

level0 = _define_token(0)
level1 = _define_token(1)
level2 = _define_token(3)
level3 = _define_token(6)
level4 = _define_token(8)



"""Sample of using the checkbox."""

from material_ui._component import Component
from material_ui.checkbox import Checkbox
from material_ui.layout_basics import Row
from material_ui.tokens import md_sys_color
from qtpy.QtCore import QMargins
from qtpy.QtWidgets import QApplication


class SampleCheckbox(Component):
    def __init__(self) -> None:
        super().__init__()

        self.sx = {"background-color": md_sys_color.background}

        row = Row()
        row.gap = 30
        row.margins = QMargins(40, 30, 40, 30)

        checkbox1 = Checkbox()
        row.add_widget(checkbox1)

        checkbox2 = Checkbox()
        checkbox2.selected = True
        row.add_widget(checkbox2)

        checkbox3 = Checkbox()
        checkbox3.indeterminate = True
        row.add_widget(checkbox3)

        self.overlay_widget(row)


def main() -> None:
    app = QApplication()
    window = SampleCheckbox()
    window.show()
    app.exec_()


if __name__ == "__main__":
    main()



"""Design tokens for md.comp.menu."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

cascading_menu_indicator_icon_color = _define_token("md.sys.color.on-surface-variant")
cascading_menu_indicator_icon_size = _define_token(24)
container_color = _define_token("md.sys.color.surface-container")
container_elevation = _define_token("md.sys.elevation.level2")
container_shadow_color = _define_token("md.sys.color.shadow")
container_shape = _define_token("md.sys.shape.corner.extra-small")
container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
divider_color = _define_token("md.sys.color.surface-variant")
divider_height = _define_token(1)
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.inner-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
list_item_container_height = _define_token(48)
list_item_disabled_label_text_color = _define_token("md.sys.color.on-surface")
list_item_disabled_label_text_opacity = _define_token(0.38)
list_item_focus_label_text_color = _define_token("md.sys.color.on-surface")
list_item_focus_state_layer_color = _define_token("md.sys.color.on-surface")
list_item_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
list_item_hover_label_text_color = _define_token("md.sys.color.on-surface")
list_item_hover_state_layer_color = _define_token("md.sys.color.on-surface")
list_item_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
list_item_label_text_color = _define_token("md.sys.color.on-surface")
list_item_label_text_font = _define_token("md.sys.typescale.label-large.font")
list_item_label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
list_item_label_text_size = _define_token("md.sys.typescale.label-large.size")
list_item_label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
list_item_label_text_weight = _define_token("md.sys.typescale.label-large.weight")
list_item_pressed_label_text_color = _define_token("md.sys.color.on-surface")
list_item_pressed_state_layer_color = _define_token("md.sys.color.on-surface")
list_item_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
list_item_selected_container_color = _define_token("md.sys.color.secondary-container")
list_item_selected_label_text_color = _define_token("md.sys.color.on-secondary-container")
list_item_selected_with_leading_icon_leading_icon_color = _define_token("md.sys.color.on-secondary-container")
list_item_selected_with_leading_icon_trailing_icon_color = _define_token("md.sys.color.on-secondary-container")
list_item_with_leading_icon_disabled_leading_icon_color = _define_token("md.sys.color.on-surface")
list_item_with_leading_icon_disabled_leading_icon_opacity = _define_token(0.38)
list_item_with_leading_icon_focus_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_with_leading_icon_hover_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_with_leading_icon_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_with_leading_icon_leading_icon_size = _define_token(24)
list_item_with_leading_icon_pressed_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_with_trailing_icon_disabled_trailing_icon_color = _define_token("md.sys.color.on-surface")
list_item_with_trailing_icon_disabled_trailing_icon_opacity = _define_token(0.38)
list_item_with_trailing_icon_focus_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_with_trailing_icon_hover_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_with_trailing_icon_pressed_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_with_trailing_icon_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
list_item_with_trailing_icon_trailing_icon_size = _define_token(24)



from pytestqt.qtbot import QtBot
from qtpy.QtCore import QEasingCurve
from qtpy.QtWidgets import QWidget

from material_ui._component import State, _TransitionConfig


def test_State_transition_applies_over_time(qtbot: QtBot):
    duration_ms = 50
    start_value = 10.0
    end_value = 20.0

    state = State(start_value, "")
    state.set_transition(_TransitionConfig(duration_ms, QEasingCurve.Type.Linear))
    state.set_value(end_value)

    periodically_inspected_values: list[float] = []
    for _ in range(duration_ms // 5 + 1):
        periodically_inspected_values.append(state.get_value())
        qtbot.wait(duration_ms // 5)

    assert state.get_value() == end_value
    assert periodically_inspected_values == sorted(periodically_inspected_values)
    assert min(periodically_inspected_values) >= start_value
    assert max(periodically_inspected_values) <= end_value


def test_State_transition_instant_changes_take_last_value(
    qtbot: QtBot,
):
    state = State(10.0, "")
    state.set_transition(_TransitionConfig(50, QEasingCurve.Type.Linear))
    state.set_value(20.0)
    state.set_value(10.0)
    qtbot.wait(60)
    assert state.get_value() == 10.0


def test_State_animate_to_instant_changes_take_last_value(
    qtbot: QtBot,
):
    state = State(10.0, "")
    state.animate_to(20.0, 50, QEasingCurve.Type.Linear)
    state.animate_to(10.0, 50, QEasingCurve.Type.Linear)
    qtbot.wait(60)
    assert state.get_value() == 10.0


def test_State___repr___current_value_and_parent_type(qtbot: QtBot):
    widget = QWidget()
    qtbot.add_widget(widget)
    state = State(1, "state")
    state.setParent(widget)
    assert repr(state) == "<State 'state' of component 'QWidget' (current value: 1)>"


def test_State___repr___null_parent():
    state = State(1, "state")
    assert repr(state) == "<State 'state' of component 'no-parent' (current value: 1)>"


def test_State_bound_cant_be_set():
    state1 = State(1, "")
    state2 = State(2, "")
    assert state2.get_value() == 2
    state2.bind(state1)
    assert state2.get_value() == 1
    state2.set_value(3)
    assert state2.get_value() == 1

    state1.set_value(4)
    assert state2.get_value() == 4



"""Tests for material_ui.tokens._utils.py."""

import pytest
from pytest_mock import MockerFixture
from qtpy.QtGui import QColor

from material_ui.theming.theme_hook import ThemeHook
from material_ui.tokens import (
    md_comp_switch,
    md_ref_palette,
    md_sys_color,
    md_sys_elevation,
    md_sys_shape,
)
from material_ui.tokens._utils import (
    DesignToken,
    _is_indirection,
    _resolve_indirection,
    find_root_token,
    override_token,
    resolve_token,
    to_python_name,
)


def test_DesignToken_hash_basic_tests() -> None:
    assert hash(md_sys_color.primary) != hash(md_ref_palette.primary40)
    assert hash(md_ref_palette.primary40) == hash(md_ref_palette.primary40)
    assert hash(md_ref_palette.primary40) != hash(md_ref_palette.primary30)


def test_DesignToken_hash_root_tokens_match() -> None:
    # Check the hash matches after resolving the indirection.
    assert hash(md_sys_elevation.level1) != hash(md_comp_switch.handle_elevation)
    assert hash(md_sys_elevation.level1) == hash(
        find_root_token(md_comp_switch.handle_elevation),
    )
    # The main use case is for using tokens as keys in a dict.
    config = {md_sys_elevation.level1: "foo"}
    result = config[find_root_token(md_comp_switch.handle_elevation)]
    assert result == "foo"


def test_resolve_token_direct_value() -> None:
    assert resolve_token(md_ref_palette.primary40) == QColor("#6750a4")


def test_resolve_token_indirection_1_level() -> None:
    assert resolve_token(md_sys_color.primary) == QColor("#6750a4")


def test_resolve_token_indirection_2_levels() -> None:
    assert resolve_token(md_comp_switch.focus_indicator_thickness) == 3


def test_resolve_token_invalid_indirection() -> None:
    with pytest.raises(ValueError):
        resolve_token(DesignToken("invalid_name"))


def test__resolve_indirection_invalid_module() -> None:
    assert _resolve_indirection("invalid_name") is None


def test__resolve_indirection_valid_module_invalid_name() -> None:
    assert _resolve_indirection("md.sys.color.invalid_name") is None


def test_find_root_token_itself() -> None:
    assert find_root_token(md_ref_palette.error10) == md_ref_palette.error10


def test_find_root_token_indirection() -> None:
    assert find_root_token(md_comp_switch.handle_elevation) == md_sys_elevation.level1


def test__is_indirection_str_enum() -> None:
    # TODO: remove this and use properly typed DesignTokens instead of
    #   str for Indirections
    assert not _is_indirection(md_sys_shape.corner_full)


def test_to_python_name() -> None:
    result = to_python_name("md.comp.elevated-button.container-color")
    assert result == "md_comp_elevated_button_container_color"


def test_override_token_value_token(mocker: MockerFixture):
    stub = mocker.stub()
    ThemeHook.get().on_change.connect(stub)

    override_token(md_sys_color.background, QColor("#ff0000"))
    assert stub.call_count == 1
    assert resolve_token(md_sys_color.background) == QColor("#ff0000")


def test_override_token_indirection(mocker: MockerFixture):
    stub = mocker.stub()
    ThemeHook.get().on_change.connect(stub)

    assert resolve_token(md_comp_switch.handle_elevation) == 1

    override_token(md_sys_elevation.level1, 10)
    assert stub.call_count == 1
    assert resolve_token(md_comp_switch.handle_elevation) == 10



"""Design tokens for md.comp.navigation-rail."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

active_indicator_color = _define_token("md.sys.color.secondary-container")
active_indicator_height = _define_token(32)
active_indicator_shape = _define_token("md.sys.shape.corner.full")
active_indicator_width = _define_token(56)
active_focus_icon_color = _define_token("md.sys.color.on-secondary-container")
active_focus_label_text_color = _define_token("md.sys.color.on-surface")
active_focus_state_layer_color = _define_token("md.sys.color.on-surface")
active_hover_icon_color = _define_token("md.sys.color.on-secondary-container")
active_hover_label_text_color = _define_token("md.sys.color.on-surface")
active_hover_state_layer_color = _define_token("md.sys.color.on-surface")
active_icon_color = _define_token("md.sys.color.on-secondary-container")
active_label_text_color = _define_token("md.sys.color.on-surface")
active_label_text_weight = _define_token("md.sys.typescale.label-medium.weight.prominent")
active_pressed_icon_color = _define_token("md.sys.color.on-secondary-container")
active_pressed_label_text_color = _define_token("md.sys.color.on-surface")
active_pressed_state_layer_color = _define_token("md.sys.color.on-surface")
badge_color = _define_token("md.sys.color.error")
badge_shape = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
badge_size = _define_token(6)
container_color = _define_token("md.sys.color.surface")
container_elevation = _define_token("md.sys.elevation.level0")
container_shape = _define_token("md.sys.shape.corner.none")
container_width = _define_token(80)
focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
icon_size = _define_token(24)
inactive_focus_icon_color = _define_token("md.sys.color.on-surface")
inactive_focus_label_text_color = _define_token("md.sys.color.on-surface")
inactive_focus_state_layer_color = _define_token("md.sys.color.on-surface")
inactive_hover_icon_color = _define_token("md.sys.color.on-surface")
inactive_hover_label_text_color = _define_token("md.sys.color.on-surface")
inactive_hover_state_layer_color = _define_token("md.sys.color.on-surface")
inactive_icon_color = _define_token("md.sys.color.on-surface-variant")
inactive_label_text_color = _define_token("md.sys.color.on-surface-variant")
inactive_pressed_icon_color = _define_token("md.sys.color.on-surface")
inactive_pressed_label_text_color = _define_token("md.sys.color.on-surface")
inactive_pressed_state_layer_color = _define_token("md.sys.color.on-surface")
label_text_font = _define_token("md.sys.typescale.label-medium.font")
label_text_font_family = _define_token("md.sys.typescale.label-medium.font")
label_text_font_size = _define_token("md.sys.typescale.label-medium.size")
label_text_letter_spacing = _define_token("md.sys.typescale.label-medium.tracking")
label_text_line_height = _define_token("md.sys.typescale.label-medium.line-height")
label_text_size = _define_token("md.sys.typescale.label-medium.size")
label_text_tracking = _define_token("md.sys.typescale.label-medium.tracking")
label_text_weight = _define_token("md.sys.typescale.label-medium.weight")
large_badge_label_color = _define_token("md.sys.color.on-error")
large_badge_label_font = _define_token("md.sys.typescale.label-small.font")
large_badge_label_font_family = _define_token("md.sys.typescale.label-small.font")
large_badge_label_line_height = _define_token("md.sys.typescale.label-small.line-height")
large_badge_label_size = _define_token("md.sys.typescale.label-small.size")
large_badge_label_tracking = _define_token("md.sys.typescale.label-small.tracking")
large_badge_label_weight = _define_token("md.sys.typescale.label-small.weight")
large_badge_color = _define_token("md.sys.color.error")
large_badge_shape = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
large_badge_size = _define_token(16)
menu_focus_icon_color = _define_token("md.sys.color.on-surface")
menu_focus_state_layer_color = _define_token("md.sys.color.on-surface")
menu_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
menu_hover_icon_color = _define_token("md.sys.color.on-surface")
menu_hover_state_layer_color = _define_token("md.sys.color.on-surface")
menu_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
menu_icon_color = _define_token("md.sys.color.on-surface-variant")
menu_icon_size = _define_token(24)
menu_pressed_icon_color = _define_token("md.sys.color.on-surface")
menu_pressed_state_layer_color = _define_token("md.sys.color.on-surface")
menu_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
no_label_active_indicator_height = _define_token(56)
no_label_active_indicator_shape = _define_token("md.sys.shape.corner.full")
pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")



"""Experimental file for figuring things out."""

from material_ui.tokens import DesignToken, resolve_token
from qtpy.QtWidgets import QGraphicsDropShadowEffect
from qtpy.QtGui import QColor
from qtpy.QtCore import QPropertyAnimation, QPointF
from material_ui.tokens import md_sys_elevation, md_sys_color
from material_ui.tokens._utils import find_root_token

_ELEVATION_BLUR_RADIUS_MAP: dict[DesignToken, int] = {
    md_sys_elevation.level0: 0,
    md_sys_elevation.level1: 3,
    md_sys_elevation.level2: 6,
    md_sys_elevation.level3: 8,
    md_sys_elevation.level4: 10,
    # TODO: implement token generator overrides for missing tokens
    # md_sys_elevation.level5: 4,
}

_ELEVATION_OFFSET_MAP: dict[DesignToken, QPointF] = {
    md_sys_elevation.level0: QPointF(0.0, 0.0),
    md_sys_elevation.level1: QPointF(0.0, 1.0),
    md_sys_elevation.level2: QPointF(0.0, 2.0),
    md_sys_elevation.level3: QPointF(0.0, 4.0),
    md_sys_elevation.level4: QPointF(0.0, 6.0),
    # md_sys_elevation.level5: QPointF(0, 4),
}


class DropShadow(QGraphicsDropShadowEffect):
    """Drop shadow supporting Material design tokens.

    TODO:
      transition for elevation
      2 shadows per elevation level - key and ambient
        see: https://github.com/material-components/material-web/blob/main/elevation/internal/_elevation.scss#L63
    """

    def __init__(self) -> None:
        super().__init__()

        self._elevation = md_sys_elevation.level0
        self._shadow_color = md_sys_color.shadow
        self._apply_elevation()

    @property
    def elevation(self) -> DesignToken:
        """Get the elevation value."""
        return self._elevation

    @elevation.setter
    def elevation(self, value: DesignToken) -> None:
        """Set the elevation value."""
        if self._elevation == value:
            return
        self._elevation = value
        self._apply_elevation()

    def _apply_elevation(self) -> None:
        self.setBlurRadius(self._get_computed_blur_radius())
        self.setOffset(self._get_computed_offset())
        # self.setBlurRadius(0)
        # self.setOffset(1, 1)

    def _get_computed_blur_radius(self) -> int:
        return _ELEVATION_BLUR_RADIUS_MAP[find_root_token(self._elevation)]

    def _get_computed_offset(self) -> QPointF:
        return _ELEVATION_OFFSET_MAP[find_root_token(self._elevation)]

    def animate_elevation_to(self, value: DesignToken) -> None:
        """Animate the elevation to a new value."""
        self._elevation = value

        blur_radius_animation = QPropertyAnimation()
        blur_radius_animation.setParent(self)
        blur_radius_animation.setTargetObject(self)
        blur_radius_animation.setPropertyName(b"blurRadius")
        blur_radius_animation.setEndValue(self._get_computed_blur_radius())
        blur_radius_animation.setDuration(120)
        # blur_radius_animation.setEasingCurve(QEasingCurve.InCubic)
        blur_radius_animation.start()

        offset_animation = QPropertyAnimation()
        offset_animation.setParent(self)
        offset_animation.setTargetObject(self)
        offset_animation.setPropertyName(b"offset")
        offset_animation.setEndValue(self._get_computed_offset())
        offset_animation.setDuration(120)
        # offset_animation.setEasingCurve(QEasingCurve.InCubic)
        offset_animation.start()

    @property
    def shadow_color(self) -> DesignToken:
        """Get the shadow color value."""
        return self._shadow_color

    @shadow_color.setter
    def shadow_color(self, value: DesignToken) -> None:
        """Set the shadow color value."""
        if self._shadow_color == value:
            return
        self._shadow_color = value

        resolved_color = resolve_token(self._shadow_color)
        if not isinstance(resolved_color, QColor):
            raise RuntimeError(
                f"Unexpected shadow_color token value (expected QColor, got "
                f"{type(resolved_color).__name__})"
            )
        resolved_color.setAlphaF(0.6)
        self.setColor(resolved_color)
        # self.setColor(QColor("rgba(0, 244, 0, 70)"))


# Testing for a multi shadow effect based on CSS
# also focus indicator outside widget bounds?
from qtpy.QtGui import QPainter, QTransform
from qtpy.QtWidgets import QGraphicsEffect
from qtpy.QtCore import QRectF

X = 2


class MyGraphicsEffect(QGraphicsEffect):
    """Testing Effect."""

    def boundingRectFor(self, rect: QRectF) -> QRectF:
        """Override the bounding rectangle to add extra padding."""
        return rect.adjusted(-X, -X, X, X)

    def draw(self, painter: QPainter) -> None:
        """Override the draw method to add custom drawing."""
        # return
        # self.drawSource(painter)
        # return
        # if not painter.isActive():
        #     return
        #     painter.begin(painter.device())
        dpr = painter.device().devicePixelRatioF()
        # painter.save()
        print(dpr)
        # painter.scale(1/dpr, 1/dpr)
        # original_transform = painter.worldTransform()
        # painter.setWorldTransform(QTransform())
        painter.setBrush(QColor(255, 0, 0, 50))
        painter.drawRect(self.boundingRect().adjusted(-X, -X, X, X))
        # painter.drawRect(self.boundingRect().adjusted(-X*dpr, -X*dpr, X*dpr, X*dpr))
        # painter.restore()
        # from qtpy.QtGui import QTransform
        # painter.setTransform(QTransform())
        self.drawSource(painter)
        # painter.setWorldTransform(original_transform)


from qtpy.QtWidgets import QApplication, QWidget


def test_graphics_effect():
    app = QApplication()
    window = QWidget()
    window.setStyleSheet("background-color:white;")
    window.resize(200, 200)
    widget = QWidget(window)
    widget.setGeometry(50, 50, 100, 100)
    widget.setStyleSheet("background-color:lightblue;border-radius:10px;")
    effect = MyGraphicsEffect()
    widget.setGraphicsEffect(effect)
    window.show()
    app.exec_()


def test_icon_font():
    app = QApplication()

    import httpx
    import tempfile
    from pathlib import Path

    font_file = (
        Path(tempfile.gettempdir()) / "MaterialSymbolsOutlined[FILL,GRAD,opsz,wght].ttf"
    )
    if not font_file.exists():
        resp = httpx.get(
            "https://raw.githubusercontent.com/google/material-design-icons/refs/heads/master/variablefont/MaterialSymbolsOutlined[FILL,GRAD,opsz,wght].ttf"
        )
        with open(font_file, "wb") as f:
            f.write(resp.content)

    from qtpy.QtGui import QFontDatabase, QFont

    font_id = QFontDatabase.addApplicationFont(str(font_file))
    family = QFontDatabase.applicationFontFamilies(font_id)
    print(family)
    # font = QFontDatabase.font(family[0], "100", 12)
    font = QFont("Material Symbols Outlined", pointSize=24, weight=400)
    # font.setVariableAxis(QFont.Tag("FILL"), 1)
    font.setVariableAxis(QFont.Tag("GRAD"), 200)
    # font.variableAxisTags()
    # font.setStyleHint()
    from qtpy.QtWidgets import QLabel
    label = QLabel("home")
    label.setFont(font)
    # label.font().setStyleStrategy(QFont.PreferAntialias)
    label.setStyleSheet("background-color:blue;color:green;")
    label.show()

    app.exec()


if __name__ == "__main__":
    # test_graphics_effect()
    test_icon_font()



"""Checkbox component."""

from typing import cast

from qtpy.QtCore import QEasingCurve, QPointF, Qt
from qtpy.QtGui import QColor, QLinearGradient
from qtpy.QtWidgets import QGraphicsOpacityEffect

from material_ui._component import Component, Signal, effect, use_state
from material_ui.icon import Icon
from material_ui.ripple import Ripple
from material_ui.shape import Shape
from material_ui.tokens import md_comp_checkbox as tokens
from material_ui.tokens._utils import resolve_token


class Checkbox(Component):
    """Checkbox component."""

    selected = use_state(False)
    """Whether the checkbox is checked."""

    indeterminate = use_state(False)
    """Whether the checkbox is in an indeterminate state."""

    on_change: Signal[bool]
    """Emitted when the user toggles the checkbox."""

    _outline_width = use_state(tokens.unselected_outline_width)
    _container_fill_opacity = use_state(1.0)
    _icon_name = use_state("check")
    _ripple_origin = use_state(cast("QPointF | None", None))
    _state_layer_color = use_state(tokens.unselected_hover_state_layer_color)
    _tick_fade_in_value = use_state(
        0.0,
        transition=400,
        easing=QEasingCurve.Type.InQuad,
    )

    def __init__(self) -> None:
        super().__init__()

        self.setFixedSize(48, 48)
        self.clicked.connect(self._on_clicked)
        self.should_propagate_click = False
        self.setCursor(Qt.CursorShape.PointingHandCursor)

        self._container = Shape()
        self._container.corner_shape = tokens.container_shape
        self._container.setFixedSize(
            cast("int", resolve_token(tokens.container_height)),
            cast("int", resolve_token(tokens.container_width)),
        )
        self._container.color = tokens.selected_container_color
        self._container.opacity = self._container_fill_opacity
        self._container.setParent(self)
        self._container.move(15, 15)

        ripple = Ripple()
        ripple.ripple_origin = self._ripple_origin
        ripple.color = tokens.unselected_pressed_state_layer_color
        ripple.setParent(self)
        ripple.setFixedSize(
            cast("int", resolve_token(tokens.state_layer_size)),
            cast("int", resolve_token(tokens.state_layer_size)),
        )
        ripple.move(4, 4)

        state_layer = Shape()
        state_layer.setParent(self)
        state_layer.move(4, 4)
        state_layer.setFixedSize(
            cast("int", resolve_token(tokens.state_layer_size)),
            cast("int", resolve_token(tokens.state_layer_size)),
        )
        state_layer.corner_shape = tokens.state_layer_shape
        state_layer.color = self._state_layer_color
        state_layer.opacity = tokens.unselected_hover_state_layer_opacity
        state_layer.visible = self.hovered

        icon = Icon()
        icon.icon_name = self._icon_name
        icon.font_size = 14
        icon.weight = 600
        icon.color = tokens.selected_icon_color
        self._icon_opacity_effect = QGraphicsOpacityEffect()
        # Set opacity at 1 so only the mask has effect (default is 0.7).
        self._icon_opacity_effect.setOpacity(1.0)
        self._icon_opacity_effect.setParent(icon)
        icon.setGraphicsEffect(self._icon_opacity_effect)
        self._container.overlay_widget(icon, center=True)

    def _on_clicked(self) -> None:
        if self.indeterminate:
            self.indeterminate = False
            self.selected = True
        else:
            self.selected = not self.selected
        self.on_change.emit(not self.selected)

    @effect(selected, indeterminate)
    def _apply_main_visual_states(self) -> None:
        self._tick_fade_in_value = 1.0 if self.selected else 0.0
        self._outline_width = (
            tokens.selected_outline_width
            if self.selected or self.indeterminate
            else tokens.unselected_outline_width
        )
        self._container_fill_opacity = (
            1.0 if self.selected or self.indeterminate else 0.0
        )
        self._icon_name = (
            "check_indeterminate_small"
            if self.indeterminate
            else "check"
            if self.selected
            else ""
        )

    @effect(Component.pressed)
    def _set_ripple_origin(self) -> None:
        if self.pressed:
            self._ripple_origin = QPointF(self.width() / 2 - 2, self.height() / 2 - 2)
        else:
            self._ripple_origin = None

    @effect(_tick_fade_in_value, indeterminate)
    def _apply_icon_opacity_mask(self) -> None:
        if self.indeterminate or self._tick_fade_in_value == 1.0:
            self._icon_opacity_effect.setOpacityMask(QColor("white"))
            return
        grad = QLinearGradient()
        grad.setStart(0, 0)
        grad.setFinalStop(18, 0)
        grad.setColorAt(self._tick_fade_in_value, "white")
        grad.setColorAt(1, "transparent")
        self._icon_opacity_effect.setOpacityMask(grad)

    @effect(_outline_width)
    def _apply_outline_width(self) -> None:
        self._container.sx = {
            **self._container.sx,
            "border-width": self._outline_width,
            "border-color": tokens.unselected_outline_color,
            "border-style": "solid",
        }



"""Ripple (for pressing)."""

from typing import cast

from qtpy.QtCore import QEasingCurve, QPointF, Qt
from qtpy.QtGui import QColor, QPainter, QPainterPath, QPaintEvent

from material_ui._component import Component, effect, use_state
from material_ui.tokens import md_sys_color, md_sys_state
from material_ui.tokens._utils import resolve_token


class Ripple(Component):
    """Ripple to overlay on a widget when pressed."""

    ripple_origin = use_state(cast("QPointF | None", None))
    color = use_state(md_sys_color.primary)
    opacity = use_state(md_sys_state.pressed_state_layer_opacity)

    clip_half_rounded = use_state(True)
    """Whether to clip the ripple to a half-rounded rectangle."""

    _opacity_value = use_state(0.0)
    _draw_origin = use_state(QPointF(0, 0))
    _scale = use_state(0.0)

    @effect(_opacity_value, _draw_origin, _scale)
    def _update_widget(self) -> None:
        # Tell Qt to call paintEvent.
        self.update()

    @effect(ripple_origin)
    def _animate_ripple_values(self) -> None:
        origin = self.ripple_origin
        if origin is None:
            # Fade out the ripple when the origin is reset. I.e., when
            # the button is released.
            self._find_state("_opacity_value").animate_to(
                0.0,
                800,
                QEasingCurve.Type.OutCubic,
            )
            return
        self._draw_origin = QPointF(origin)  # copy not bind
        self._find_state("_opacity_value").animate_to(
            resolve_token(self.opacity),
            50,
            QEasingCurve.Type.OutCubic,
        )
        self._scale = 1.0
        ripple_total_scale = max(self.width(), self.height()) * 2
        self._find_state("_scale").animate_to(
            ripple_total_scale,
            1200,
            QEasingCurve.Type.OutCubic,
        )

    def paintEvent(self, event: QPaintEvent) -> None:  # noqa: N802
        """Draw the ripple effect."""
        super().paintEvent(event)
        painter = QPainter(self)
        if self.clip_half_rounded:
            clip_path = QPainterPath()
            half_size = min(self.width(), self.height()) // 2
            clip_path.addRoundedRect(self.rect(), half_size, half_size)
            painter.setClipPath(clip_path)
        painter.setPen(Qt.PenStyle.NoPen)
        color = QColor(resolve_token(self.color))
        color.setAlphaF(self._opacity_value)
        painter.setBrush(color)
        painter.drawEllipse(self._draw_origin, self._scale, self._scale)
        painter.end()



"""Design tokens for md.comp.search-bar."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

avatar_shape = _define_token("md.sys.shape.corner.full")
avatar_size = _define_token(30)
container_color = _define_token("md.sys.color.surface-container-high")
container_elevation = _define_token("md.sys.elevation.level3")
container_height = _define_token(56)
container_shape = _define_token("md.sys.shape.corner.full")
container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
hover_state_layer_color = _define_token("md.sys.color.on-surface")
hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
hover_supporting_text_color = _define_token("md.sys.color.on-surface-variant")
input_text_color = _define_token("md.sys.color.on-surface")
input_text_font = _define_token("md.sys.typescale.body-large.font")
input_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
input_text_size = _define_token("md.sys.typescale.body-large.size")
input_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
input_text_weight = _define_token("md.sys.typescale.body-large.weight")
leading_icon_color = _define_token("md.sys.color.on-surface")
pressed_state_layer_color = _define_token("md.sys.color.on-surface")
pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
pressed_supporting_text_color = _define_token("md.sys.color.on-surface-variant")
supporting_text_color = _define_token("md.sys.color.on-surface-variant")
supporting_text_font = _define_token("md.sys.typescale.body-large.font")
supporting_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
supporting_text_size = _define_token("md.sys.typescale.body-large.size")
supporting_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
supporting_text_weight = _define_token("md.sys.typescale.body-large.weight")
trailing_icon_color = _define_token("md.sys.color.on-surface-variant")



"""Design tokens for md.comp.top-app-bar."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

large_container_color = _define_token("md.sys.color.surface")
large_container_elevation = _define_token("md.sys.elevation.level0")
large_container_height = _define_token(152)
large_container_shape = _define_token("md.sys.shape.corner.none")
large_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
large_headline_color = _define_token("md.sys.color.on-surface")
large_headline_font = _define_token("md.sys.typescale.headline-medium.font")
large_headline_line_height = _define_token("md.sys.typescale.headline-medium.line-height")
large_headline_size = _define_token("md.sys.typescale.headline-medium.size")
large_headline_tracking = _define_token("md.sys.typescale.headline-medium.tracking")
large_headline_weight = _define_token("md.sys.typescale.headline-medium.weight")
large_leading_icon_color = _define_token("md.sys.color.on-surface")
large_leading_icon_size = _define_token(24)
large_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
large_trailing_icon_size = _define_token(24)
medium_container_color = _define_token("md.sys.color.surface")
medium_container_elevation = _define_token("md.sys.elevation.level0")
medium_container_height = _define_token(112)
medium_container_shape = _define_token("md.sys.shape.corner.none")
medium_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
medium_headline_color = _define_token("md.sys.color.on-surface")
medium_headline_font = _define_token("md.sys.typescale.headline-small.font")
medium_headline_line_height = _define_token("md.sys.typescale.headline-small.line-height")
medium_headline_size = _define_token("md.sys.typescale.headline-small.size")
medium_headline_tracking = _define_token("md.sys.typescale.headline-small.tracking")
medium_headline_weight = _define_token("md.sys.typescale.headline-small.weight")
medium_leading_icon_color = _define_token("md.sys.color.on-surface")
medium_leading_icon_size = _define_token(24)
medium_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
medium_trailing_icon_size = _define_token(24)
small_centered_avatar_shape = _define_token("md.sys.shape.corner.full")
small_centered_avatar_size = _define_token(30)
small_centered_container_color = _define_token("md.sys.color.surface")
small_centered_container_elevation = _define_token("md.sys.elevation.level0")
small_centered_container_height = _define_token(64)
small_centered_container_shape = _define_token("md.sys.shape.corner.none")
small_centered_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
small_centered_headline_color = _define_token("md.sys.color.on-surface")
small_centered_headline_font = _define_token("md.sys.typescale.title-large.font")
small_centered_headline_line_height = _define_token("md.sys.typescale.title-large.line-height")
small_centered_headline_size = _define_token("md.sys.typescale.title-large.size")
small_centered_headline_tracking = _define_token("md.sys.typescale.title-large.tracking")
small_centered_headline_weight = _define_token("md.sys.typescale.title-large.weight")
small_centered_leading_icon_color = _define_token("md.sys.color.on-surface")
small_centered_leading_icon_size = _define_token(24)
small_centered_on_scroll_container_color = _define_token("md.sys.color.surface-container")
small_centered_on_scroll_container_elevation = _define_token("md.sys.elevation.level2")
small_centered_on_scroll_container_shadow_color = _define_token("md.sys.color.shadow")
small_centered_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
small_centered_trailing_icon_size = _define_token(24)
small_container_color = _define_token("md.sys.color.surface")
small_container_elevation = _define_token("md.sys.elevation.level0")
small_container_height = _define_token(64)
small_container_shape = _define_token("md.sys.shape.corner.none")
small_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
small_headline_color = _define_token("md.sys.color.on-surface")
small_headline_font = _define_token("md.sys.typescale.title-large.font")
small_headline_line_height = _define_token("md.sys.typescale.title-large.line-height")
small_headline_size = _define_token("md.sys.typescale.title-large.size")
small_headline_tracking = _define_token("md.sys.typescale.title-large.tracking")
small_headline_weight = _define_token("md.sys.typescale.title-large.weight")
small_leading_icon_color = _define_token("md.sys.color.on-surface")
small_leading_icon_size = _define_token(24)
small_on_scroll_container_color = _define_token("md.sys.color.surface-container")
small_on_scroll_container_elevation = _define_token("md.sys.elevation.level2")
small_on_scroll_container_shadow_color = _define_token("md.sys.color.shadow")
small_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
small_trailing_icon_size = _define_token(24)



"""Design tokens for md.comp.full-screen-dialog."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

action_bar_container_color = _define_token("md.sys.color.surface")
action_bar_container_elevation = _define_token("md.sys.elevation.level0")
action_bar_container_height = _define_token(56)
action_bar_focus_label_text_color = _define_token("md.sys.color.primary")
action_bar_focus_state_layer_color = _define_token("md.sys.color.primary")
action_bar_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
action_bar_hover_label_text_color = _define_token("md.sys.color.primary")
action_bar_hover_state_layer_color = _define_token("md.sys.color.primary")
action_bar_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
action_bar_label_text_color = _define_token("md.sys.color.primary")
action_bar_label_text_font = _define_token("md.sys.typescale.label-large.font")
action_bar_label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
action_bar_label_text_size = _define_token("md.sys.typescale.label-large.size")
action_bar_label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
action_bar_label_text_weight = _define_token("md.sys.typescale.label-large.weight")
action_bar_on_scroll_container_elevation = _define_token("md.sys.elevation.level2")
action_bar_pressed_label_text_color = _define_token("md.sys.color.primary")
action_bar_pressed_state_layer_color = _define_token("md.sys.color.primary")
action_bar_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
container_color = _define_token("md.sys.color.surface")
container_elevation = _define_token("md.sys.elevation.level0")
container_shape = _define_token("md.sys.shape.corner.none")
header_action_focus_label_text_color = _define_token("md.sys.color.primary")
header_action_focus_state_layer_color = _define_token("md.sys.color.primary")
header_action_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
header_action_hover_label_text_color = _define_token("md.sys.color.primary")
header_action_hover_state_layer_color = _define_token("md.sys.color.primary")
header_action_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
header_action_label_text_color = _define_token("md.sys.color.primary")
header_action_label_text_font = _define_token("md.sys.typescale.label-large.font")
header_action_label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
header_action_label_text_size = _define_token("md.sys.typescale.label-large.size")
header_action_label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
header_action_label_text_weight = _define_token("md.sys.typescale.label-large.weight")
header_action_pressed_label_text_color = _define_token("md.sys.color.primary")
header_action_pressed_state_layer_color = _define_token("md.sys.color.primary")
header_action_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
header_container_color = _define_token("md.sys.color.surface")
header_container_elevation = _define_token("md.sys.elevation.level0")
header_container_height = _define_token(56)
header_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
header_headline_color = _define_token("md.sys.color.on-surface")
header_headline_font = _define_token("md.sys.typescale.title-large.font")
header_headline_line_height = _define_token("md.sys.typescale.title-large.line-height")
header_headline_size = _define_token("md.sys.typescale.title-large.size")
header_headline_tracking = _define_token("md.sys.typescale.title-large.tracking")
header_headline_weight = _define_token("md.sys.typescale.title-large.weight")
header_icon_color = _define_token("md.sys.color.on-surface")
header_icon_size = _define_token(24)
header_on_scroll_container_color = _define_token("md.sys.color.surface-container")
header_on_scroll_container_elevation = _define_token("md.sys.elevation.level2")
with_divider_divider_color = _define_token("md.sys.color.surface-variant")
with_divider_divider_height = _define_token(1)



"""Design tokens for md.comp.input-chip."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

container_elevation = _define_token("md.sys.elevation.level0")
container_height = _define_token(32)
container_shape = _define_token("md.sys.shape.corner.small")
disabled_label_text_color = _define_token("md.sys.color.on-surface")
disabled_label_text_opacity = _define_token(0.38)
disabled_outline_color = _define_token("md.sys.color.on-surface")
disabled_outline_opacity = _define_token(0.12)
disabled_selected_container_color = _define_token("md.sys.color.on-surface")
disabled_selected_container_opacity = _define_token(0.12)
disabled_unselected_outline_color = _define_token("md.sys.color.on-surface")
disabled_unselected_outline_opacity = _define_token(0.12)
dragged_container_elevation = _define_token("md.sys.elevation.level4")
dragged_label_text_color = _define_token("md.sys.color.on-surface-variant")
dragged_state_layer_color = _define_token("md.sys.color.on-surface-variant")
dragged_state_layer_opacity = _define_token("md.sys.state.dragged.state-layer-opacity")
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
focus_label_text_color = _define_token("md.sys.color.on-surface-variant")
focus_outline_color = _define_token("md.sys.color.on-surface-variant")
focus_state_layer_color = _define_token("md.sys.color.on-surface-variant")
focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
hover_label_text_color = _define_token("md.sys.color.on-surface-variant")
hover_state_layer_color = _define_token("md.sys.color.on-surface-variant")
hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
label_text_color = _define_token("md.sys.color.on-surface-variant")
label_text_font = _define_token("md.sys.typescale.label-large.font")
label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
label_text_size = _define_token("md.sys.typescale.label-large.size")
label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
label_text_weight = _define_token("md.sys.typescale.label-large.weight")
outline_color = _define_token("md.sys.color.outline")
outline_width = _define_token(1)
pressed_label_text_color = _define_token("md.sys.color.on-surface-variant")
pressed_state_layer_color = _define_token("md.sys.color.on-surface-variant")
pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
selected_container_color = _define_token("md.sys.color.secondary-container")
selected_dragged_label_text_color = _define_token("md.sys.color.on-secondary-container")
selected_dragged_state_layer_color = _define_token("md.sys.color.on-secondary-container")
selected_dragged_state_layer_opacity = _define_token("md.sys.state.dragged.state-layer-opacity")
selected_focus_label_text_color = _define_token("md.sys.color.on-secondary-container")
selected_focus_state_layer_color = _define_token("md.sys.color.on-secondary-container")
selected_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
selected_hover_label_text_color = _define_token("md.sys.color.on-secondary-container")
selected_hover_state_layer_color = _define_token("md.sys.color.on-secondary-container")
selected_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
selected_label_text_color = _define_token("md.sys.color.on-secondary-container")
selected_outline_width = _define_token(0)
selected_pressed_label_text_color = _define_token("md.sys.color.on-secondary-container")
selected_pressed_state_layer_color = _define_token("md.sys.color.on-secondary-container")
selected_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
unselected_dragged_label_text_color = _define_token("md.sys.color.on-surface-variant")
unselected_dragged_state_layer_color = _define_token("md.sys.color.on-surface-variant")
unselected_dragged_state_layer_opacity = _define_token("md.sys.state.dragged.state-layer-opacity")
unselected_focus_label_text_color = _define_token("md.sys.color.on-surface-variant")
unselected_focus_outline_color = _define_token("md.sys.color.on-surface-variant")
unselected_focus_state_layer_color = _define_token("md.sys.color.on-surface-variant")
unselected_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
unselected_hover_label_text_color = _define_token("md.sys.color.on-surface-variant")
unselected_hover_state_layer_color = _define_token("md.sys.color.on-surface-variant")
unselected_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
unselected_label_text_color = _define_token("md.sys.color.on-surface-variant")
unselected_outline_color = _define_token("md.sys.color.outline-variant")
unselected_outline_width = _define_token(1)
unselected_pressed_label_text_color = _define_token("md.sys.color.on-surface-variant")
unselected_pressed_state_layer_color = _define_token("md.sys.color.on-surface-variant")
unselected_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
with_avatar_avatar_shape = _define_token("md.sys.shape.corner.full")
with_avatar_avatar_size = _define_token(24)
with_avatar_disabled_avatar_opacity = _define_token(0.38)
with_leading_icon_disabled_leading_icon_color = _define_token("md.sys.color.on-surface")
with_leading_icon_disabled_leading_icon_opacity = _define_token(0.38)
with_leading_icon_dragged_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
with_leading_icon_focus_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
with_leading_icon_hover_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
with_leading_icon_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
with_leading_icon_leading_icon_size = _define_token(18)
with_leading_icon_pressed_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
with_leading_icon_selected_dragged_leading_icon_color = _define_token("md.sys.color.on-secondary-container")
with_leading_icon_selected_focus_leading_icon_color = _define_token("md.sys.color.primary")
with_leading_icon_selected_hover_leading_icon_color = _define_token("md.sys.color.primary")
with_leading_icon_selected_leading_icon_color = _define_token("md.sys.color.primary")
with_leading_icon_selected_pressed_leading_icon_color = _define_token("md.sys.color.primary")
with_leading_icon_unselected_dragged_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
with_leading_icon_unselected_focus_leading_icon_color = _define_token("md.sys.color.primary")
with_leading_icon_unselected_hover_leading_icon_color = _define_token("md.sys.color.primary")
with_leading_icon_unselected_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
with_leading_icon_unselected_pressed_leading_icon_color = _define_token("md.sys.color.primary")
with_trailing_icon_disabled_trailing_icon_color = _define_token("md.sys.color.on-surface")
with_trailing_icon_disabled_trailing_icon_opacity = _define_token(0.38)
with_trailing_icon_dragged_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
with_trailing_icon_focus_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
with_trailing_icon_hover_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
with_trailing_icon_pressed_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
with_trailing_icon_selected_dragged_trailing_icon_color = _define_token("md.sys.color.primary")
with_trailing_icon_selected_focus_trailing_icon_color = _define_token("md.sys.color.on-secondary-container")
with_trailing_icon_selected_hover_trailing_icon_color = _define_token("md.sys.color.on-secondary-container")
with_trailing_icon_selected_pressed_trailing_icon_color = _define_token("md.sys.color.on-secondary-container")
with_trailing_icon_selected_trailing_icon_color = _define_token("md.sys.color.on-secondary-container")
with_trailing_icon_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
with_trailing_icon_trailing_icon_size = _define_token(18)
with_trailing_icon_unselected_dragged_trailing_icon_color = _define_token("md.sys.color.primary")
with_trailing_icon_unselected_focus_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
with_trailing_icon_unselected_hover_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
with_trailing_icon_unselected_pressed_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
with_trailing_icon_unselected_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")



"""Sample of changing colors with design tokens theming system."""

from material_ui._component import Component
from qtpy.QtWidgets import QApplication


class ChangingColors(Component):
    def __init__(self) -> None:
        super().__init__()


# Color in hue, chroma, tone form
from materialyoucolor.hct import Hct

# There are 9 different variants of scheme.
from materialyoucolor.scheme.scheme_tonal_spot import SchemeTonalSpot

# Others you can import: SchemeExpressive, SchemeFruitSalad, SchemeMonochrome, SchemeRainbow, SchemeVibrant, SchemeNeutral, SchemeFidelity and SchemeContent

# SchemeTonalSpot is android default
scheme = SchemeTonalSpot(
    Hct.from_int(0xFF4181EE),
    is_dark=False,
    contrast_level=0.0,
)


def main() -> None:
    app = QApplication()
    window = ChangingColors()
    window.show()
    app.exec_()


if __name__ == "__main__":
    main()
